# Copyright 2014 San Francisco Estuary Institute.

# This file is part of RipZET.
#
# RipZET is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# RipZET is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with RipZET.  If not, see http://www.gnu.org/licenses/.


import arcpy
from arcpy.sa import *
import sys
import os
from os.path import abspath
from os.path import dirname

sys.path.insert(0, abspath(os.path.join(dirname('__file__'), 'common')))
sys.path.insert(0, abspath(os.path.join(dirname('__file__'), 'RipZET_lib')))

from common.sfei_utils import parameter

import RipZET_lib.ripzet_globals as rz_globals
import RipZET_Prep as prep
import RipZET_Hillslope as hillslope
import RipZET_Vegetation as vegetation
import RipZET_RiparianZones as riparianZones
import RipZET_HyConCrossSections as hyConCrossSections
import RipZET_HyConZones as hyConZones
#import RipZET_ThalwegLine as thalwegLine


class Toolbox(object):
    def __init__(self):
        self.label = "RipZET Tools"
        self.alias = "ripzet"
        # List of tool classes associated with this toolbox
        self.tools = [rzPrep, rzHillslope, rzVegetation, rzRiparianZones, rzHyConCrossSections, rzHyConZones]#, rzThalwegLine]



#------------------------------------------------------------------------------
# Prep
#------------------------------------------------------------------------------
class rzPrep(object):
    def __init__(self):
        """Define the tool (tool name is the name of the class)."""
        self.label = "1 - Prep"
        self.canRunInBackground = True
        self.multiVal = (None, None)

        rz_globals.initializePaths(os.path.dirname(os.path.abspath(__file__)))

        workspace = parameter("Output Directory", "workspace", "DEFolder")
        fcIn = parameter("Stream or Wetland Features", "fcIn", "GPFeatureLayer")
        fcIn.filter.list = ["Polyline", "Polygon"]
        typeField = parameter("Type Field", "typeField", "Field", enabled=False)
        typeField.parameterDependencies = [fcIn.name]
        strOrdField = parameter("Stream Order Field", "strOrdField", "Field", "Optional", enabled=False)
        strOrdField.parameterDependencies = [fcIn.name]
        staBnkDist = parameter("Standard Bank Distances (m)", "staBnkDist", "GPString", "Optional", enabled=False)
        strChannel = parameter("Natural Channel Type", "strChannel", "GPString", "Optional", enabled=False)
        strDitch = parameter("Unnatural Channel Type", "strDitch", "GPString", "Optional", enabled=False)
        openWaterCode = parameter("Open Water Type", "openWaterCode", "GPString", "Optional", enabled=False)
        openWaterCode.parameterDependencies = [fcIn.name]
        nonRiparian = parameter("Non-Riparian Types", "nonRiparian", "GPString", "Optional", multiValue=True, enabled=False)
        nonRiparian.filter.list = []
        innerRiparian = parameter("Inner Riparian Types", "innerRiparian", "GPString", "Optional", multiValue=True, enabled=False)
        innerRiparian.filter.list = []
        excludeRiparian = parameter("Exclude Riparian Types", "excludeRiparian", "GPString", "Optional", multiValue=True, enabled=False)
        excludeRiparian.filter.list = []
        preserveFields = parameter("Preserve Fields in Output", "preserveFields", "Field", "Optional", multiValue=True, enabled=False)
        preserveFields.parameterDependencies = [fcIn.name]

        # initialize parameter list
        self.parameters = [workspace,
                           fcIn,
                           typeField,
                           strOrdField,
                           staBnkDist,
                           strChannel,
                           strDitch,
                           openWaterCode,
                           nonRiparian,
                           innerRiparian,
                           excludeRiparian,
                           preserveFields]


    def getParameterInfo(self):
        return self.parameters


    def updateParameters(self, params):
        """Modify the values and properties of parameters before internal
        validation is performed.  This method is called whenever a parameter
        has been changed."""
        shapeType = None

        # populate dict from list for easier referencing
        # strings provide better self-documentation than list indices
        parameterDict = dict([(p.name, p) for p in params])

        enabled = []
        disabled = []

        if parameterDict["fcIn"].value:
            shapeType = arcpy.Describe(parameterDict["fcIn"].value).shapeType
            if shapeType == 'Polyline':
                enabled = ["typeField", "strOrdField", "staBnkDist", "strChannel", "strDitch"]
                disabled = ["openWaterCode", "nonRiparian", "innerRiparian", "excludeRiparian"]
                # set errors
                if not parameterDict['strOrdField'].value:
                    parameterDict['strOrdField'].setErrorMessage("The stream order field is required for stream polylines.")
                else:
                    parameterDict['strOrdField'].clearMessage()

            elif shapeType == 'Polygon':
                enabled = ["typeField", "openWaterCode", "nonRiparian", "innerRiparian", "excludeRiparian"]
                disabled = ["strOrdField", "staBnkDist", "strChannel", "strDitch"]
                # clear errors
                parameterDict["strOrdField"].clearMessage()

            enabled.append("preserveFields")

            for p in enabled:
                parameterDict[p].enabled = True
            for p in disabled:
                parameterDict[p].enabled = False
        else:
            disabled = ["typeField", "strOrdField", "staBnkDist", "strChannel", "strDitch", "openWaterCode", "nonRiparian",
                        "innerRiparian", "excludeRiparian", "preserveFields"]
            for p in disabled:
                parameterDict[p].enabled = False
                parameterDict[p].value = None

        # Populate stream and ditch values
        if shapeType == 'Polyline' and parameterDict["typeField"].altered and parameterDict["typeField"].value:
            fc, col = str(parameterDict["fcIn"].value), str(parameterDict["typeField"].value)
            if self.multiVal != (fc, col):
                self.multiVal = (fc, col)
                vals = sorted(set([row[0] for row in arcpy.da.SearchCursor(fc, [col]) if row[0]])) # ignore empty strings
                parameterDict["strChannel"].filter.list = vals
                parameterDict["strDitch"].filter.list = vals
                if parameterDict["strChannel"].value not in parameterDict["strChannel"].filter.list:
                    parameterDict["strChannel"].value = parameterDict["strChannel"].filter.list[0]
                if parameterDict["strDitch"].value not in parameterDict["strDitch"].filter.list:
                    if len(parameterDict["strDitch"].filter.list) > 1:
                        parameterDict["strDitch"].value = parameterDict["strDitch"].filter.list[1]
                    else:
                        parameterDict["strDitch"].value = parameterDict["strDitch"].filter.list[0]

        # Populate nonRiparian and innerRiparian values
        if shapeType == 'Polygon' and parameterDict["typeField"].altered:
            fc, col = str(parameterDict["fcIn"].value), str(parameterDict["typeField"].value)
            if self.multiVal != (fc, col):
                self.multiVal = (fc, col)
                vals = sorted(set([row[0] for row in arcpy.da.SearchCursor(fc, [col]) if row[0]])) # ignore empty strings
                parameterDict["nonRiparian"].filter.list = vals
                parameterDict["innerRiparian"].filter.list = vals
                parameterDict["excludeRiparian"].filter.list = vals

        # for now, use default distances
        useDefaultBankDistances = True

        # default bank distance values
        defaultBankDistanceMin = 0.5
        defaultBankDistanceMax = 2.0

        # Fill out default staBankDist with (1, .. max stream order) incremented by 0.5
        # with 1 at end for ditch value
        if shapeType == 'Polyline' and parameterDict["strOrdField"].value and not parameterDict["staBnkDist"].value:
            fc = parameterDict["fcIn"].value
            strOrdField = str(parameterDict["strOrdField"].value)
            maxStrOrd = max([row[0] for row in arcpy.da.SearchCursor(fc, [strOrdField])])

            if useDefaultBankDistances:
                bankDistanceMin = defaultBankDistanceMin
                bankDistanceMax = defaultBankDistanceMax
            else:
                bankDistanceMin = 1.0
                bankDistanceMax = maxStrOrd

            staBankDists = []
            i = bankDistanceMin
            for x in range(1, maxStrOrd + 1):
                staBankDists.append(i)
                if i < bankDistanceMax:
                    i += 0.5
            # set last value (ditch) to 1.0
            staBankDists.append(1.0)
            parameterDict["staBnkDist"].value = str(staBankDists).strip('[]')

        # refresh self.parameters and local params to ensure they reflect any updates
        self.parameters = list(parameterDict.values())
        params[:] = []
        params.extend(self.parameters)

        return


    def updateMessages(self, params):
        """Modify the messages created by internal validation for each tool
        parameter.  This method is called after internal validation."""

        # populate dict from list for easier referencing
        parameterDict = dict([(p.name, p) for p in params])


    def execute(self, params, messages):
        """The source code of the tool."""
        return prep.execute(params, messages)


#------------------------------------------------------------------------------
# Hillslope
#------------------------------------------------------------------------------
class rzHillslope(object):
    def __init__(self):
        """Define the tool (tool name is the name of the class)."""
        self.label = "2 - Hillslope"
        self.canRunInBackground = True

        rz_globals.initializePaths(os.path.dirname(os.path.abspath(__file__)))

        tempGDB = parameter("RipZET Work Geodatabase", "tempGDB", "DEWorkspace")
        slpIn = parameter("Percent Slope Raster", "slpIn", "GPRasterLayer")
        slpLimit = parameter("Percent Slope Threshold", "slpLimit", "Long", defaultValue=20)
        slpSegDist = parameter("Slope Segment Distance (m)", "slpSegDist", "Long", defaultValue=100)
        lineClusTol = parameter("Line Cluster Tolerance (m)", "lineClusTol", "Long", defaultValue=25)
        mode = parameter("Tool Mode", "mode", "GPString")
        mode.filter.list = ["Streams", "Wetlands"]

        # initialize parameter list
        self.parameters = [tempGDB,
                           slpIn,
                           slpLimit,
                           slpSegDist,
                           lineClusTol,
                           mode]


    def getParameterInfo(self):
        """Define parameter definitions"""
        return self.parameters


    def isLicensed(self):
        """Set whether tool is licensed to execute."""
        try:
            if arcpy.CheckExtension("Spatial") == "Available":
                arcpy.CheckOutExtension("Spatial")
            else:
                raise Exception

        except:
            return False

        return True


    def updateParameters(self, params):
        """Modify the values and properties of parameters before internal
        validation is performed.  This method is called whenever a parameter
        has been changed."""

        return


    def updateMessages(self, params):
        """Modify the messages created by internal validation for each tool
        parameter.  This method is called after internal validation."""
        return


    def execute(self, params, messages):
        """The source code of the tool."""
        return hillslope.execute(params, messages)


#------------------------------------------------------------------------------
# Vegetation
#------------------------------------------------------------------------------
class rzVegetation(object):
    def __init__(self):
        """Define the tool (tool name is the name of the class)."""
        self.label = "3 - Vegetation"
        self.canRunInBackground = True

        rz_globals.initializePaths(os.path.dirname(os.path.abspath(__file__)))

        tempGDB = parameter("RipZET Work Geodatabase", "tempGDB", "DEWorkspace")
        veg = parameter("Vegetation Features", "veg", "GPFeatureLayer")
        lineClusTol = parameter("Line Cluster Tolerance (m)", "lineClusTol", "Long", defaultValue=25)
        treeHtField = parameter("Tree Height Field", "treeHtField", "Field")
        treeHtField.parameterDependencies = [veg.name]
        treeHtFactor = parameter("Tree Height Factor", "treeHtFactor", "Long", defaultValue=2)
        stanBufDistField = parameter("Standard Zone Distance Field", "stanBufDistField", "Field")
        stanBufDistField.parameterDependencies = [veg.name]
        vegCodeField = parameter("Vegetation Code Field", "vegCodeField", "Field")
        vegCodeField.parameterDependencies = [veg.name]
        mode = parameter("Tool Mode", "mode", "GPString")
        mode.filter.list = ["Streams", "Wetlands"]

        self.parameters = [tempGDB,
                           veg,
                           lineClusTol,
                           treeHtField,
                           treeHtFactor,
                           stanBufDistField,
                           vegCodeField,
                           mode]


    def getParameterInfo(self):
        """Define parameter definitions"""

        return self.parameters


    def isLicensed(self):
        """Set whether tool is licensed to execute."""
        try:
            if arcpy.CheckExtension("Spatial") == "Available":
                arcpy.CheckOutExtension("Spatial")
            else:
                raise Exception

        except:
            return False

        return True


    def updateParameters(self, params):
        """Modify the values and properties of parameters before internal
        validation is performed.  This method is called whenever a parameter
        has been changed."""

        return


    def updateMessages(self, parameters):
        """Modify the messages created by internal validation for each tool
        parameter.  This method is called after internal validation."""
        return


    def execute(self, params, messages):
        """The source code of the tool."""
        return vegetation.execute(params, messages)


#------------------------------------------------------------------------------
# RiparianZones
#------------------------------------------------------------------------------
class rzRiparianZones(object):
    def __init__(self):
        """Define the tool (tool name is the name of the class)."""
        self.label = "4 - Riparian Zones"
        self.canRunInBackground = True

        rz_globals.initializePaths(os.path.dirname(os.path.abspath(__file__)))

        tempGDB = parameter("RipZET Work Geodatabase", "tempGDB", "DEWorkspace")
        ripType = parameter("Riparian Zone Type", "ripType", "GPString")
        ripType.filter.list = ["Slope", "Vegetation"]
        mode = parameter("Tool Mode", "mode", "GPString")
        mode.filter.list = ["Streams", "Wetlands"]
        ripBufBins = parameter("Riparian Zone Bins (m)", "ripBufBins", "GPString", defaultValue="0-10,10-30,30-50,50-100,100-")
        ripLineBins = parameter("Riparian Line Bins (m)", "ripLineBins", "GPString", defaultValue="0-50,50-100,100-200,200-500,500-")
        innerRipBufDist = parameter("Inner Riparian Zone Distance (m)", "innerRipBufDist", "Double", "Optional", defaultValue=1)

        self.parameters = [tempGDB,
                           ripType,
                           mode,
                           ripBufBins,
                           ripLineBins,
                           innerRipBufDist] 


    def getParameterInfo(self):
        """Define parameter definitions"""

        return self.parameters


    def isLicensed(self):
        """Set whether tool is licensed to execute."""
        try:
            if arcpy.CheckExtension("Spatial") == "Available":
                arcpy.CheckOutExtension("Spatial")
            else:
                raise Exception

        except:
            return False

        return True


    def updateParameters(self, params):
        """Modify the values and properties of parameters before internal
        validation is performed.  This method is called whenever a parameter
        has been changed."""

        parameterDict = dict([(p.name, p) for p in params])

        if parameterDict["mode"].altered:
            if parameterDict["mode"].valueAsText.lower() == "wetlands":
                parameterDict["innerRipBufDist"].enabled = True
            elif parameterDict["mode"].valueAsText.lower() == "streams":
                parameterDict["innerRipBufDist"].enabled = False

        return


    def updateMessages(self, params):
        """Modify the messages created by internal validation for each tool
        parameter.  This method is called after internal validation."""

        return


    def execute(self, params, messages):
        """The source code of the tool."""

        return riparianZones.execute(params, messages)


#------------------------------------------------------------------------------
# HyConTransects
#------------------------------------------------------------------------------
class rzHyConCrossSections(object):
    def __init__(self):
        """Define the tool (tool name is the name of the class)."""
        self.label = "HyCon 1 - {0}s".format(rz_globals.transectDisplayName().title())
        self.canRunInBackground = True

        rz_globals.initializePaths(os.path.dirname(os.path.abspath(__file__)))

        siteName = parameter("Site Name", "siteName", "GPString")

        toolMode = parameter("Tool Mode", "toolMode", "GPString")
        toolMode.filter.list = rz_globals.TOOL_MODES
        toolMode.value = toolMode.filter.list[0]

        workspace = parameter("Workspace or RipZET Work Geodatabase", "workspace", "DEWorkspace", "Optional")
        sitePt = parameter("Site Point", "sitePt", "GPFeatureLayer", "Optional")
        streams = parameter("Streams", "streams", "GPFeatureLayer", "Optional")
        watersheds = parameter("Watersheds", "watersheds", "GPFeatureLayer", "Optional")
        elevationRaster = parameter("LIDAR Raster", "elevationRaster", "GPRasterLayer", "Optional")
        clipElevationRaster = parameter("Clip LIDAR", "clipElevationRaster", "GPBoolean", defaultValue=True)
        hydCurveEquation = parameter("Hydraulic Curve Equation", "hydCurveEquation", "GPString", "Derived")
        hydCurveEquation.filter.list = ['South-West Arizona', 'Coastal Oregon', 'SF Bay', 'Tahoe']
        equationDisplay = parameter("", "equationDisplay", "GPString", "Derived", defaultValue="[Hydraulic Curve Equation Display]", enabled=False)

        transectLocOption = parameter("{0} Mode".format(toolMode.filter.list[1]), "transectLocOption", "GPString", "Optional", enabled=False)
        transectLocs = parameter("{0} Locations".format(rz_globals.transectDisplayName().title()), "transectLocs", "GPFeatureLayer", "Optional", enabled=False)
        transectLocOption.filter.list = rz_globals.TRANSECT_LOC_OPTIONS
        transectLocOption.value = transectLocOption.filter.list[0]

        # hard-code curve selection for now
        hydCurveEquation.enabled = False
        hydCurveEquation.value = 'SF Bay'
        equationDisplay.value = 'BF = 1.0195 * DA ^ 0.3687'

        self.parameters = [siteName,
                           toolMode,
                           workspace,
                           sitePt,
                           streams,
                           watersheds,
                           elevationRaster,
                           clipElevationRaster,
                           transectLocOption,
                           transectLocs,
                           hydCurveEquation,
                           equationDisplay]


    def getParameterInfo(self):
        """Define parameter definitions"""

        return self.parameters


    def isLicensed(self):
        """Set whether tool is licensed to execute."""
        try:
            if arcpy.CheckExtension("Spatial") == "Available":
                arcpy.CheckOutExtension("Spatial")
            else:
                raise Exception

            if arcpy.CheckExtension("3D") == "Available":
                arcpy.CheckOutExtension("3D")
            else:
                raise Exception
        except:
            return False

        return True


    def updateParameters(self, params):
        """Modify the values and properties of parameters before internal
        validation is performed.  This method is called whenever a parameter
        has been changed."""

        parameterDict = dict([(p.name, p) for p in params])

        # Fill in equationDisplay param with formula based on hydCurveEquation param selection
        if parameterDict["hydCurveEquation"].altered:
            if parameterDict["hydCurveEquation"].value == 'South-West Arizona':
                parameterDict["equationDisplay"].value = 'BF = 0.7812 * DA ^ 0.219'
            elif parameterDict["hydCurveEquation"].value == 'Coastal Oregon':
                parameterDict["equationDisplay"].value = 'BF = 0.9 * DA ^ 0.389'
            elif parameterDict["hydCurveEquation"].value == 'SF Bay':
                parameterDict["equationDisplay"].value = 'BF = 1.0195 * DA ^ 0.3687'

        # enable/disable parameters based on mode
        if parameterDict["toolMode"].altered:
            enabled = []
            disabled = []

            if parameterDict["toolMode"].value == parameterDict["toolMode"].filter.list[0]:
                enabled = ["workspace", "sitePt", "streams", "elevationRaster", "watersheds", "clipElevationRaster"]
                disabled = ["transectLocs", "transectLocOption"]
            else:
                enabled = ["transectLocs", "transectLocOption"]
                disabled = ["workspace", "sitePt", "streams", "elevationRaster", "watersheds", "clipElevationRaster"]

            for p in enabled:
                parameterDict[p].enabled = True
            for p in disabled:
                parameterDict[p].enabled = False

        return


    def updateMessages(self, params):
        """Modify the messages created by internal validation for each tool
        parameter.  This method is called after internal validation."""

        # populate dict from list for easier referencing
        parameterDict = dict([(p.name, p) for p in params])

        return


    def execute(self, params, messages):
        """The source code of the tool."""
        tool = hyConCrossSections.HyConCrossSections(params)
        return tool.execute()


#------------------------------------------------------------------------------
# HyConZones
#------------------------------------------------------------------------------
class rzHyConZones(object):
    def __init__(self):
        """Define the tool (tool name is the name of the class)."""
        self.label = "HyCon 2 - Zones"
        self.canRunInBackground = True

        rz_globals.initializePaths(os.path.dirname(os.path.abspath(__file__)))


        tempGdb = parameter("RipZET Work Geodatabase", "tempGdb", "DEWorkspace")
        xlFile = parameter("HyCon {0}s XLSM File".format(rz_globals.transectDisplayName().title()), "xlFile", "DEFile")

        self.parameters = [tempGdb,
                           xlFile]

    def getParameterInfo(self):
        """Define parameter definitions"""
        return self.parameters

    def isLicensed(self):
        """Set whether tool is licensed to execute."""
        try:
            if arcpy.CheckExtension("Spatial") == "Available":
                arcpy.CheckOutExtension("Spatial")
            else:
                raise Exception
        except:
            return False

        return True

    def updateParameters(self, params):
        """Modify the values and properties of parameters before internal
        validation is performed.  This method is called whenever a parameter
        has been changed."""

        parameterDict = dict([(p.name, p) for p in params])

        return

    def updateMessages(self, params):
        """Modify the messages created by internal validation for each tool
        parameter.  This method is called after internal validation."""

        parameterDict = dict([(p.name, p) for p in params])

        return

    def execute(self, params, messages):
        """The source code of the tool."""
        tool = hyConZones.HyConZonesTool(params)
        return tool.execute()


#------------------------------------------------------------------------------
# ThalwegLines
#------------------------------------------------------------------------------
# class rzThalwegLine(object):
#     def __init__(self):
#         """Define the tool (tool name is the name of the class)."""
#         self.label = "Thalweg Line"
#         self.canRunInBackground = True
#
#         rz_globals.initializePaths(os.path.dirname(os.path.abspath(__file__)))
#
#         streams = parameter("Stream Line", "streams", "GPFeatureLayer")
#         outputLocation = parameter("Output Location", "outputLocation", "DEWorkspace")
#         outputName = parameter("Output Name", "outputName", "GPString")
#         elevationRaster = parameter("Elevation Raster", "elevationRaster", "GPRasterLayer")
#         searchDistance = parameter("Search Distance", "searchDistance", "GPDouble", defaultValue=10)
#         sampleInterval = parameter("Sample Interval", "sampleInterval", "GPDouble")
#         intervalFactor = parameter("Interval Factor", "intervalFactor", "GPLong", defaultValue=4)
#         output3d = parameter("Output 3D", "output3d", "GPBoolean", defaultValue=False)
#
#         self.parameters = [streams,
#                            outputLocation,
#                            outputName,
#                            elevationRaster,
#                            searchDistance,
#                            sampleInterval,
#                            intervalFactor,
#                            output3d]
#
#     def getParameterInfo(self):
#         """Define parameter definitions"""
#         return self.parameters
#
#     def isLicensed(self):
#         """Set whether tool is licensed to execute."""
#         try:
#             if arcpy.CheckExtension("Spatial") == "Available":
#                 arcpy.CheckOutExtension("Spatial")
#             else:
#                 raise Exception
#         except:
#             return False
#
#         return True
#
#     def updateParameters(self, params):
#         """Modify the values and properties of parameters before internal
#         validation is performed.  This method is called whenever a parameter
#         has been changed."""
#
#         parameterDict = dict([(p.name, p) for p in params])
#
#         if parameterDict["outputName"].altered and parameterDict["outputName"].value:
#             if parameterDict["outputLocation"].value:
#                 baseName = parameterDict["outputName"].valueAsText
#                 workspace = parameterDict["outputLocation"].valueAsText
#                 parameterDict["outputName"].value = os.path.basename(arcpy.CreateUniqueName(os.path.basename(arcpy.ValidateTableName(baseName, workspace)), workspace))
#         elif parameterDict["streams"].altered and parameterDict["streams"].value:
#             baseName = os.path.basename(parameterDict["streams"].valueAsText) + "_thalweg"
#             if not parameterDict["outputLocation"].value:
#                 parameterDict["outputName"].value = baseName
#             else:
#                 workspace = parameterDict["outputLocation"].valueAsText
#                 parameterDict["outputName"].value = os.path.basename(arcpy.CreateUniqueName(os.path.basename(arcpy.ValidateTableName(baseName, workspace)), workspace))
#         elif parameterDict["outputLocation"].altered and parameterDict["outputLocation"].value and parameterDict["streams"].value:
#             baseName = os.path.basename(parameterDict["streams"].valueAsText) + "_thalweg"
#             workspace = parameterDict["outputLocation"].valueAsText
#             parameterDict["outputName"].value = os.path.basename(arcpy.CreateUniqueName(os.path.basename(arcpy.ValidateTableName(baseName, workspace)), workspace))
#
#         # if parameterDict["elevationRaster"].altered:
#         #     if parameterDict["elevationRaster"].value:
#         #         try:
#         #             raster = Raster(parameterDict["elevationRaster"].valueAsText)
#         #             cellSize = (raster.meanCellHeight + raster.meanCellWidth) / 2
#         #             parameterDict["sampleInterval"].value = cellSize * 4
#         #         except:
#         #             pass
#
#         return
#
#     def updateMessages(self, params):
#         """Modify the messages created by internal validation for each tool
#         parameter.  This method is called after internal validation."""
#
#         parameterDict = dict([(p.name, p) for p in params])
#
#         return
#
#     def execute(self, params, messages):
#         """The source code of the tool."""
#         thalwegLine.execute(params, messages)